@@ -22,6 +22,7 @@
 
 #include <KDecoration2/DecoratedClient>
 #include <KColorUtils>
+#include <KIconLoader>
 
 #include <QPainter>
 #include <QVariantAnimation>
@@ -143,8 +144,20 @@ namespace Breeze
         {
 
             const QRectF iconRect( geometry().topLeft(), m_iconSize );
-            decoration()->client().data()->icon().paint(painter, iconRect.toRect());
-
+            if (auto deco =  qobject_cast<Decoration*>(decoration())) {
+                const QPalette activePalette = KIconLoader::global()->customPalette();
+                QPalette palette = decoration()->client().data()->palette();
+                palette.setColor(QPalette::Foreground, deco->fontColor());
+                KIconLoader::global()->setCustomPalette(palette);
+                decoration()->client().data()->icon().paint(painter, iconRect.toRect());
+                if (activePalette == QPalette()) {
+                    KIconLoader::global()->resetPalette();
+                }    else {
+                    KIconLoader::global()->setCustomPalette(palette);
+                }
+            } else {
+                decoration()->client().data()->icon().paint(painter, iconRect.toRect());
+            }
 
         } else {
 
