@@ -70,11 +70,9 @@ namespace Breeze
             { setEnabled( false ); }
             break;
 
-            #if QT_VERSION >= 0x050000
             case QEvent::ApplicationStateChange:
             { setEnabled( false ); }
             break;
-            #endif
 
             default: break;
 
