@@ -1,6 +1,6 @@
 /*************************************************************************
  * Copyright (C) 2014 by Hugo Pereira Da Costa <hugo.pereira@free.fr>    *
- * Copyright (C) 2018 by Vlad Zagorodniy <vlad.zahorodnii@kde.org>       *
+ * Copyright (C) 2018, 2020 by Vlad Zahorodnii <vlad.zahorodnii@kde.org> *
  *                                                                       *
  * This program is free software; you can redistribute it and/or modify  *
  * it under the terms of the GNU General Public License as published by  *
@@ -32,22 +32,10 @@
 #include <QMenu>
 #include <QPainter>
 #include <QPixmap>
+#include <QPlatformSurfaceEvent>
 #include <QToolBar>
 #include <QTextStream>
 
-#if BREEZE_HAVE_X11
-#include <QX11Info>
-#endif
-
-#if BREEZE_HAVE_KWAYLAND
-#include <KWayland/Client/buffer.h>
-#include <KWayland/Client/connection_thread.h>
-#include <KWayland/Client/registry.h>
-#include <KWayland/Client/shadow.h>
-#include <KWayland/Client/shm_pool.h>
-#include <KWayland/Client/surface.h>
-#endif
-
 namespace
 {
     using Breeze::CompositeShadowParams;
@@ -82,8 +70,6 @@ namespace
 namespace Breeze
 {
 
-    const char ShadowHelper::netWMShadowAtomName[] ="_KDE_NET_WM_SHADOW";
-
     //_____________________________________________________
     CompositeShadowParams ShadowHelper::lookupShadowParams(int shadowSizeEnum)
     {
@@ -111,61 +97,18 @@ namespace Breeze
         QObject( parent ),
         _helper( helper )
     {
-        // delay till event dispatcher is running as Wayland is highly async
-        QMetaObject::invokeMethod(this, "initializeWayland", Qt::QueuedConnection);
     }
 
     //_______________________________________________________
     ShadowHelper::~ShadowHelper()
     {
-
-        #if BREEZE_HAVE_X11
-        if( Helper::isX11() )
-        { foreach( const quint32& value, _pixmaps  ) xcb_free_pixmap( Helper::connection(), value ); }
-        #endif
-
-    }
-
-    //_______________________________________________________
-    void ShadowHelper::initializeWayland()
-    {
-        #if BREEZE_HAVE_KWAYLAND
-        if( !Helper::isWayland() ) return;
-
-        using namespace KWayland::Client;
-        auto connection = ConnectionThread::fromApplication( this );
-        if( !connection ) {
-            return;
-        }
-        auto registry = new Registry( connection );
-        registry->create( connection );
-        connect(registry, &Registry::interfacesAnnounced, this,
-            [registry, this] {
-                const auto interface = registry->interface( Registry::Interface::Shadow );
-                if( interface.name != 0 ) {
-                    _shadowManager = registry->createShadowManager( interface.name, interface.version, registry );
-                }
-                const auto shmInterface = registry->interface( Registry::Interface::Shm );
-                if( shmInterface.name != 0 ) {
-                    _shmPool = registry->createShmPool( shmInterface.name, shmInterface.version, registry );
-                }
-            }
-        );
-
-        registry->setup();
-        connection->roundtrip();
-        #endif
+        qDeleteAll( _shadows );
     }
 
     //______________________________________________
     void ShadowHelper::reset()
     {
-        #if BREEZE_HAVE_X11
-        if( Helper::isX11() )
-        { foreach( const quint32& value, _pixmaps  ) xcb_free_pixmap( Helper::connection(), value ); }
-        #endif
-
-        _pixmaps.clear();
+        _tiles.clear();
         _shadowTiles = TileSet();
 
     }
@@ -181,15 +124,15 @@ namespace Breeze
         { return false; }
 
         // try create shadow directly
-        if( installShadows( widget ) ) _widgets.insert( widget, widget->winId() );
-        else _widgets.insert( widget, 0 );
+        installShadows( widget );
+        _widgets.insert( widget );
 
         // install event filter
         widget->removeEventFilter( this );
         widget->installEventFilter( this );
 
         // connect destroy signal
-        connect( widget, SIGNAL(destroyed(QObject*)), SLOT(objectDeleted(QObject*)) );
+        connect( widget, &QObject::destroyed, this, &ShadowHelper::objectDeleted );
 
         return true;
 
@@ -199,7 +142,16 @@ namespace Breeze
     void ShadowHelper::unregisterWidget( QWidget* widget )
     {
         if( _widgets.remove( widget ) )
-        { uninstallShadows( widget ); }
+        {
+            // uninstall the event filter
+            widget->removeEventFilter( this );
+
+            // disconnect all signals
+            disconnect( widget, nullptr, this, nullptr );
+
+            // uninstall the shadow
+            uninstallShadows( widget );
+        }
     }
 
     //_______________________________________________________
@@ -210,42 +162,16 @@ namespace Breeze
         reset();
 
         // update property for registered widgets
-        for( QMap<QWidget*,WId>::const_iterator iter = _widgets.constBegin(); iter != _widgets.constEnd(); ++iter )
-        { installShadows( iter.key() ); }
+        for( QWidget* widget : _widgets)
+        { installShadows( widget ); }
 
     }
 
     //_______________________________________________________
     bool ShadowHelper::eventFilter( QObject* object, QEvent* event )
     {
-        if( Helper::isWayland() )
+        if( Helper::isX11() )
         {
-
-            #if BREEZE_HAVE_KWAYLAND
-            QWidget* widget( static_cast<QWidget*>( object ) );
-            if( event->type() == QEvent::Paint )
-            {
-
-                auto iter = _widgetSurfaces.constFind( widget );
-                if( iter == _widgetSurfaces.constEnd() )
-                {
-                    // install shadows and update winId
-                    installShadows( widget );
-                }
-
-            } else if( event->type() == QEvent::Hide ) {
-
-                auto iter = _widgetSurfaces.find( widget );
-                if( iter != _widgetSurfaces.end() )
-                {
-                    _widgetSurfaces.erase( iter );
-                }
-
-            }
-            #endif
-
-        } else if( Helper::isX11() ) {
-
             // check event type
             if( event->type() != QEvent::WinIdChange ) return false;
 
@@ -253,8 +179,23 @@ namespace Breeze
             QWidget* widget( static_cast<QWidget*>( object ) );
 
             // install shadows and update winId
-            if( installShadows( widget ) )
-            { _widgets.insert( widget, widget->winId() ); }
+            installShadows( widget );
+
+        } else {
+            if( event->type() != QEvent::PlatformSurface ) return false;
+
+            QWidget* widget( static_cast<QWidget*>( object ) );
+            QPlatformSurfaceEvent* surfaceEvent( static_cast<QPlatformSurfaceEvent*>( event ) );
+
+            switch( surfaceEvent->surfaceEventType() )
+            {
+                case QPlatformSurfaceEvent::SurfaceCreated:
+                    installShadows( widget );
+                    break;
+                case QPlatformSurfaceEvent::SurfaceAboutToBeDestroyed:
+                    // Don't care.
+                    break;
+            }
         }
 
         return false;
@@ -284,12 +225,7 @@ namespace Breeze
         const QSize boxSize = BoxShadowRenderer::calculateMinimumBoxSize(params.shadow1.radius)
             .expandedTo(BoxShadowRenderer::calculateMinimumBoxSize(params.shadow2.radius));
 
-        #if QT_VERSION >= 0x050300
         const qreal dpr = qApp->devicePixelRatio();
-        #else
-        const qreal dpr = 1.0;
-        #endif
-
         const qreal frameRadius = _helper.frameRadius();
 
         BoxShadowRenderer shadowRenderer;
@@ -323,11 +259,7 @@ namespace Breeze
         painter.setBrush(Qt::black);
         painter.setCompositionMode(QPainter::CompositionMode_DestinationOut);
         painter.drawRoundedRect(
-#if BREEZE_USE_KDE4
-            outerRect.adjusted(margins.left(), margins.top(), -margins.right(), -margins.bottom()),
-#else
             outerRect - margins,
-#endif
             frameRadius,
             frameRadius);
 
@@ -347,7 +279,12 @@ namespace Breeze
 
     //_______________________________________________________
     void ShadowHelper::objectDeleted( QObject* object )
-    { _widgets.remove( static_cast<QWidget*>( object ) ); }
+    {
+        QWidget* widget( static_cast<QWidget*>( object ) );
+        _widgets.remove( widget );
+        _shadows.remove( widget );
+
+    }
 
     //_______________________________________________________
     bool ShadowHelper::isMenu( QWidget* widget ) const
@@ -392,176 +329,83 @@ namespace Breeze
     }
 
     //______________________________________________
-    const QVector<quint32>& ShadowHelper::createPixmapHandles()
+    const QVector<KWindowShadowTile::Ptr>& ShadowHelper::createShadowTiles()
     {
 
-        /**
-        shadow atom and property specification available at
-        https://community.kde.org/KWin/Shadow
-        */
-
-        // create atom
-        #if BREEZE_HAVE_X11
-        if( !_atom && Helper::isX11() ) _atom = _helper.createAtom( QLatin1String( netWMShadowAtomName ) );
-        #endif
-
         // make sure size is valid
-        if( _pixmaps.empty() )
+        if( _tiles.isEmpty() )
         {
-            _pixmaps = QVector<quint32> {
-                createPixmap( _shadowTiles.pixmap( 1 ) ),
-                createPixmap( _shadowTiles.pixmap( 2 ) ),
-                createPixmap( _shadowTiles.pixmap( 5 ) ),
-                createPixmap( _shadowTiles.pixmap( 8 ) ),
-                createPixmap( _shadowTiles.pixmap( 7 ) ),
-                createPixmap( _shadowTiles.pixmap( 6 ) ),
-                createPixmap( _shadowTiles.pixmap( 3 ) ),
-                createPixmap( _shadowTiles.pixmap( 0 ) )
+            _tiles = {
+                createTile( _shadowTiles.pixmap( 1 ) ),
+                createTile( _shadowTiles.pixmap( 2 ) ),
+                createTile( _shadowTiles.pixmap( 5 ) ),
+                createTile( _shadowTiles.pixmap( 8 ) ),
+                createTile( _shadowTiles.pixmap( 7 ) ),
+                createTile( _shadowTiles.pixmap( 6 ) ),
+                createTile( _shadowTiles.pixmap( 3 ) ),
+                createTile( _shadowTiles.pixmap( 0 ) )
             };
         }
 
-        // return relevant list of pixmap handles
-        return _pixmaps;
+        // return relevant list of shadow tiles
+        return _tiles;
 
     }
 
     //______________________________________________
-    quint32 ShadowHelper::createPixmap( const QPixmap& source )
+    KWindowShadowTile::Ptr ShadowHelper::createTile( const QPixmap& source )
     {
 
-        // do nothing for invalid pixmaps
-        if( source.isNull() ) return 0;
-        if( !Helper::isX11() ) return 0;
-
-        /*
-        in some cases, pixmap handle is invalid. This is the case notably
-        when Qt uses to RasterEngine. In this case, we create an X11 Pixmap
-        explicitly and draw the source pixmap on it.
-        */
-
-        #if BREEZE_HAVE_X11
-
-        const int width( source.width() );
-        const int height( source.height() );
-
-        // create X11 pixmap
-        xcb_pixmap_t pixmap = xcb_generate_id( Helper::connection() );
-        xcb_create_pixmap( Helper::connection(), 32, pixmap, QX11Info::appRootWindow(), width, height );
-
-        // create gc
-        if( !_gc )
-        {
-            _gc = xcb_generate_id( Helper::connection() );
-            xcb_create_gc( Helper::connection(), _gc, pixmap, 0, nullptr );
-        }
-
-        // create image from QPixmap and assign to pixmap
-        QImage image( source.toImage() );
-
-
-        #if QT_VERSION >= 0x051000
-        xcb_put_image( Helper::connection(), XCB_IMAGE_FORMAT_Z_PIXMAP, pixmap, _gc, image.width(), image.height(), 0, 0, 0, 32, image.sizeInBytes(), image.constBits());
-        #else
-        xcb_put_image( Helper::connection(), XCB_IMAGE_FORMAT_Z_PIXMAP, pixmap, _gc, image.width(), image.height(), 0, 0, 0, 32, image.byteCount(), image.constBits());
-        #endif
-
-        return pixmap;
-
-        #else
-        return 0;
-        #endif
+        KWindowShadowTile::Ptr tile = KWindowShadowTile::Ptr::create();
+        tile->setImage( source.toImage() );
+        return tile;
 
     }
 
     //_______________________________________________________
-    bool ShadowHelper::installShadows( QWidget* widget )
+    void ShadowHelper::installShadows( QWidget* widget )
     {
-        if( !widget ) return false;
+        if( !widget ) return;
 
-        /*
-        From bespin code. Supposedly prevent playing with some 'pseudo-widgets'
-        that have winId matching some other -random- window
-        */
-        if( !(widget->testAttribute(Qt::WA_WState_Created) && widget->internalWinId() ))
-        { return false; }
+        // only toplevel widgets can cast drop-shadows
+        if( !widget->isWindow() ) return;
+
+        // widget must have valid native window
+        if( !widget->testAttribute( Qt::WA_WState_Created ) ) return;
 
         // create shadow tiles if needed
         shadowTiles();
+        if( !_shadowTiles.isValid() ) return;
 
-        if( !_shadowTiles.isValid() ) return false;
+        // create platform shadow tiles if needed
+        const QVector<KWindowShadowTile::Ptr>& tiles = createShadowTiles();
+        if( tiles.count() != numTiles ) return;
 
-        if( Helper::isX11() ) return installX11Shadows( widget );
-        if( Helper::isWayland() ) return installWaylandShadows( widget );
+        // find a shadow associated with the widget
+        KWindowShadow*& shadow = _shadows[ widget ];
 
-        return false;
-    }
+        // we want the shadow to be deleted after the decorated window is destroyed
+        if( !shadow )
+        {
+            shadow = new KWindowShadow( widget->windowHandle() );
 
-    //_______________________________________________________
-    bool ShadowHelper::installX11Shadows( QWidget* widget )
-    {
-        #if BREEZE_HAVE_X11
-        #ifndef QT_NO_XRENDER
+            connect( shadow, &QObject::destroyed, this, [this, widget] { _shadows.remove( widget ); } );
+        }
 
-        // create pixmap handles if needed
-        QVector<quint32> data( createPixmapHandles() );
-        if( data.size() != numPixmaps ) return false;
+        if( shadow->isCreated() )
+        { shadow->destroy(); }
 
-        const QMargins margins = shadowMargins( widget );
-        const quint32 topSize = margins.top();
-        const quint32 bottomSize = margins.bottom();
-        const quint32 leftSize( margins.left() );
-        const quint32 rightSize( margins.right() );
-
-        // assign to data and xcb property
-        data << QVector<quint32>{topSize, rightSize, bottomSize, leftSize};
-        xcb_change_property( Helper::connection(), XCB_PROP_MODE_REPLACE, widget->winId(), _atom, XCB_ATOM_CARDINAL, 32, data.size(), data.constData() );
-        xcb_flush( Helper::connection() );
-
-        return true;
-
-        #endif
-        #endif
-
-        return false;
-
-    }
-
-    //_______________________________________________________
-    bool ShadowHelper::installWaylandShadows( QWidget* widget )
-    {
-        #if BREEZE_HAVE_KWAYLAND
-        if( widget->windowHandle()->parent() ) return false;
-        if( !_shadowManager || !_shmPool ) return false;
-
-        // create shadow
-        using namespace KWayland::Client;
-        auto s = Surface::fromWindow( widget->windowHandle() );
-        if( !s ) return false;
-
-        auto shadow = _shadowManager->createShadow( s, widget );
-        if( !shadow->isValid() ) return false;
-
-        // add the shadow elements
-        shadow->attachTop( _shmPool->createBuffer( _shadowTiles.pixmap( 1 ).toImage() ) );
-        shadow->attachTopRight( _shmPool->createBuffer( _shadowTiles.pixmap( 2 ).toImage() ) );
-        shadow->attachRight( _shmPool->createBuffer( _shadowTiles.pixmap( 5 ).toImage() ) );
-        shadow->attachBottomRight( _shmPool->createBuffer( _shadowTiles.pixmap( 8 ).toImage() ) );
-        shadow->attachBottom( _shmPool->createBuffer( _shadowTiles.pixmap( 7 ).toImage() ) );
-        shadow->attachBottomLeft( _shmPool->createBuffer( _shadowTiles.pixmap( 6 ).toImage() ) );
-        shadow->attachLeft( _shmPool->createBuffer( _shadowTiles.pixmap( 3 ).toImage() ) );
-        shadow->attachTopLeft( _shmPool->createBuffer( _shadowTiles.pixmap( 0 ).toImage() ) );
-
-        shadow->setOffsets( shadowMargins( widget ) );
-        shadow->commit();
-        s->commit( Surface::CommitFlag::None );
-        _widgetSurfaces.insert(widget, s);
-
-        return true;
-        #else
-        Q_UNUSED( widget );
-        #endif
-
-        return false;
+        shadow->setTopTile( tiles[ 0 ] );
+        shadow->setTopRightTile( tiles[ 1 ] );
+        shadow->setRightTile( tiles[ 2 ] );
+        shadow->setBottomRightTile( tiles[ 3 ] );
+        shadow->setBottomTile( tiles[ 4 ] );
+        shadow->setBottomLeftTile( tiles[ 5 ] );
+        shadow->setLeftTile( tiles[ 6 ] );
+        shadow->setTopLeftTile( tiles[ 7 ] );
+        shadow->setPadding( shadowMargins( widget ) );
+        shadow->setWindow( widget->windowHandle() );
+        shadow->create();
     }
 
     //_______________________________________________________
@@ -595,14 +439,7 @@ namespace Breeze
             int bottom = widget->contentsMargins().bottom();
 
             // Need to decrement default size further due to extra hard coded round corner.
-#if BREEZE_USE_KDE4
-            margins.setLeft(margins.left() - 1);
-            margins.setTop(margins.top() - 1);
-            margins.setRight(margins.right() - 1);
-            margins.setBottom(margins.bottom() - 1);
-#else
             margins -= 1;
-#endif
 
             // Arrow can be either to the top or the bottom. Adjust margins accordingly.
             const int diff = qAbs(top - bottom);
@@ -613,54 +450,15 @@ namespace Breeze
             }
         }
 
-#if BREEZE_USE_KDE4
-        const qreal dpr = _helper.devicePixelRatio(_shadowTiles.pixmap(0));
-        margins.setLeft(margins.left() * dpr);
-        margins.setTop(margins.top() * dpr);
-        margins.setRight(margins.right() * dpr);
-        margins.setBottom(margins.bottom() * dpr);
-#else
         margins *= _helper.devicePixelRatio(_shadowTiles.pixmap(0));
-#endif
 
         return margins;
     }
 
     //_______________________________________________________
-    void ShadowHelper::uninstallShadows( QWidget* widget ) const
+    void ShadowHelper::uninstallShadows( QWidget* widget )
     {
-        if( !( widget && widget->testAttribute(Qt::WA_WState_Created) ) ) return;
-        if( Helper::isX11() ) uninstallX11Shadows( widget );
-        if( Helper::isWayland() ) uninstallWaylandShadows( widget );
-    }
-
-    //_______________________________________________________
-    void ShadowHelper::uninstallX11Shadows( QWidget* widget ) const
-    {
-        #if BREEZE_HAVE_X11
-        xcb_delete_property( Helper::connection(), widget->winId(), _atom);
-        #else
-        Q_UNUSED( widget )
-        #endif
-
-    }
-
-    //_______________________________________________________
-    void ShadowHelper::uninstallWaylandShadows( QWidget* widget ) const
-    {
-        #if BREEZE_HAVE_KWAYLAND
-        if( widget->windowHandle() && widget->windowHandle()->parent() ) return;
-        if( !_shadowManager ) return;
-
-        using namespace KWayland::Client;
-        auto s = Surface::fromWindow( widget->windowHandle() );
-        if( !s ) return;
-
-        _shadowManager->removeShadow( s );
-        s->commit( Surface::CommitFlag::None );
-        #else
-        Q_UNUSED( widget )
-        #endif
+        delete _shadows.take( widget );
     }
 
 }
